@@ -1,4 +1,4 @@
-use lightning_signer::signer::my_signer::MultiSigner;
+use lightning_signer::signer::multi_signer::MultiSigner;
 
 /// A trivial program, just to check code size
 pub fn main() {
