@@ -1,3 +1,4 @@
+use crate::prelude::*;
 use core::cmp;
 use core::convert::TryInto;
 use core::fmt;
@@ -9,34 +10,38 @@ use bitcoin::blockdata::opcodes::all::{
 };
 use bitcoin::hashes::sha256::Hash as Sha256;
 use bitcoin::hashes::{Hash, HashEngine};
-use bitcoin::secp256k1;
-use bitcoin::secp256k1::{All, Message, PublicKey, Secp256k1, Signature};
+use bitcoin::secp256k1::PublicKey;
 use bitcoin::util::address::Payload;
-use bitcoin::util::bip143;
-use bitcoin::{OutPoint, Script, SigHashType, Transaction, TxIn, TxOut};
-use lightning::chain::keysinterface::BaseSign;
+use bitcoin::{OutPoint, Script, Transaction, TxIn, TxOut};
+use lightning::chain::keysinterface::{BaseSign, InMemorySigner};
 use lightning::ln::chan_utils;
 use lightning::ln::chan_utils::{
-    get_revokeable_redeemscript, make_funding_redeemscript, HTLCOutputInCommitment, TxCreationKeys,
+    get_revokeable_redeemscript, HTLCOutputInCommitment, TxCreationKeys,
 };
 use lightning::ln::PaymentHash;
 
-use crate::node::node::ChannelSetup;
-use crate::policy::error::ValidationError;
-use crate::policy::error::ValidationError::{Mismatch, ScriptFormat, TransactionFormat};
+use crate::channel::ChannelSetup;
+use crate::policy::error::{
+    mismatch_error, script_format_error, transaction_format_error, ValidationError,
+};
 use crate::tx::script::{
     expect_data, expect_number, expect_op, expect_script_end, get_anchor_redeemscript,
     get_delayed_redeemscript, get_htlc_anchor_redeemscript,
 };
 use crate::util::crypto_utils::payload_for_p2wpkh;
 use crate::util::debug_utils::DebugPayload;
-use crate::util::enforcing_trait_impls::EnforcingSigner;
+use bitcoin::hashes::hex::ToHex;
 
 const MAX_DELAY: i64 = 1000;
-pub const ANCHOR_SAT: u64 = 330;
+/// Value for anchor outputs
+pub(crate) const ANCHOR_SAT: u64 = 330;
 
-// BEGIN NOT TESTED
-pub fn get_commitment_transaction_number_obscure_factor(
+/// Copied from lightning::ln::chan_utils where they are not public enough.
+pub(crate) const HTLC_SUCCESS_TX_WEIGHT: u64 = 703;
+/// Copied from lightning::ln::chan_utils where they are not public enough.
+pub(crate) const HTLC_TIMEOUT_TX_WEIGHT: u64 = 663;
+
+pub(crate) fn get_commitment_transaction_number_obscure_factor(
     local_payment_basepoint: &PublicKey,
     counterparty_payment_basepoint: &PublicKey,
     outbound: bool,
@@ -60,65 +65,8 @@ pub fn get_commitment_transaction_number_obscure_factor(
         | ((res[30] as u64) << 1 * 8)
         | ((res[31] as u64) << 0 * 8)
 }
-// END NOT TESTED
 
-pub fn build_close_tx(
-    to_holder_value_sat: u64,
-    to_counterparty_value_sat: u64,
-    local_shutdown_script: &Script,
-    counterparty_shutdown_script: &Script,
-    outpoint: OutPoint,
-) -> Transaction {
-    let txins = {
-        let mut ins: Vec<TxIn> = Vec::new();
-        ins.push(TxIn {
-            previous_output: outpoint,
-            script_sig: Script::new(),
-            sequence: 0xffffffff,
-            witness: Vec::new(),
-        });
-        ins
-    };
-
-    let mut txouts: Vec<(TxOut, ())> = Vec::new();
-
-    if to_counterparty_value_sat > 0 {
-        txouts.push((
-            TxOut {
-                script_pubkey: counterparty_shutdown_script.clone(),
-                value: to_counterparty_value_sat,
-            },
-            (),
-        ));
-    }
-
-    if to_holder_value_sat > 0 {
-        txouts.push((
-            TxOut {
-                script_pubkey: local_shutdown_script.clone(),
-                value: to_holder_value_sat,
-            },
-            (),
-        ));
-    }
-
-    sort_outputs(&mut txouts, |_, _| cmp::Ordering::Equal); // Ordering doesnt matter if they used our pubkey...
-
-    let mut outputs: Vec<TxOut> = Vec::new();
-    for out in txouts.drain(..) {
-        outputs.push(out.0);
-    }
-
-    Transaction {
-        version: 2,
-        lock_time: 0,
-        input: txins,
-        output: outputs,
-    }
-}
-
-// BEGIN NOT TESTED
-pub fn build_commitment_tx(
+pub(crate) fn build_commitment_tx(
     keys: &TxCreationKeys,
     info: &CommitmentInfo2,
     obscured_commitment_transaction_number: u64,
@@ -270,33 +218,11 @@ pub fn build_commitment_tx(
         htlcs,
     )
 }
-// END NOT TESTED
 
-// Sign a Bitcoin commitment tx or a mutual-close tx
-pub(crate) fn sign_commitment(
-    secp_ctx: &Secp256k1<All>,
-    keys: &EnforcingSigner,
-    counterparty_funding_pubkey: &PublicKey,
-    tx: &Transaction,
-    channel_value_sat: u64,
-) -> Result<Signature, secp256k1::Error> {
-    let funding_key = keys.funding_key();
-    let funding_pubkey = keys.pubkeys().funding_pubkey;
-    let channel_funding_redeemscript =
-        make_funding_redeemscript(&funding_pubkey, &counterparty_funding_pubkey);
-
-    let commitment_sighash = Message::from_slice(
-        &bip143::SigHashCache::new(tx).signature_hash(
-            0,
-            &channel_funding_redeemscript,
-            channel_value_sat,
-            SigHashType::All,
-        )[..],
-    )?;
-    Ok(secp_ctx.sign(&commitment_sighash, funding_key))
-}
-
-pub fn sort_outputs<T, C: Fn(&T, &T) -> cmp::Ordering>(outputs: &mut Vec<(TxOut, T)>, tie_breaker: C) {
+pub(crate) fn sort_outputs<T, C: Fn(&T, &T) -> cmp::Ordering>(
+    outputs: &mut Vec<(TxOut, T)>,
+    tie_breaker: C,
+) {
     outputs.sort_unstable_by(|a, b| {
         a.0.value.cmp(&b.0.value).then_with(|| {
             a.0.script_pubkey[..]
@@ -309,6 +235,7 @@ pub fn sort_outputs<T, C: Fn(&T, &T) -> cmp::Ordering>(outputs: &mut Vec<(TxOut,
 /// Phase 1 HTLC info
 #[derive(Clone)]
 pub struct HTLCInfo {
+    /// HTLC value
     pub value_sat: u64,
     /// RIPEMD160 of 32 bytes hash
     pub payment_hash_hash: [u8; 20],
@@ -317,29 +244,57 @@ pub struct HTLCInfo {
 }
 
 // Implement manually so we can have hex encoded payment_hash_hash.
-// BEGIN NOT TESTED
 impl fmt::Debug for HTLCInfo {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("HTLCInfo")
             .field("value_sat", &self.value_sat)
-            .field("payment_hash_hash", &hex::encode(&self.payment_hash_hash))
+            .field("payment_hash_hash", &self.payment_hash_hash.to_hex())
             .field("cltv_expiry", &self.cltv_expiry)
             .finish()
     }
 }
-// END NOT TESTED
 
 /// Phase 2 HTLC info
-#[derive(Debug, Clone)]
+#[derive(Clone, PartialEq, Eq)]
 pub struct HTLCInfo2 {
+    /// The value in satoshi
     pub value_sat: u64,
+    /// The payment hash
     pub payment_hash: PaymentHash,
     /// This is zero for offered HTLCs in phase 1
     pub cltv_expiry: u32,
 }
 
-// BEGIN NOT TESTED
-#[derive(Debug, Clone)]
+// Implement manually because PaymentHash doesn't support
+impl Ord for HTLCInfo2 {
+    fn cmp(&self, other: &Self) -> cmp::Ordering {
+        self.value_sat
+            .cmp(&other.value_sat)
+            .then_with(|| self.payment_hash.0.cmp(&other.payment_hash.0))
+            .then_with(|| self.cltv_expiry.cmp(&other.cltv_expiry))
+    }
+}
+
+// Implement manually because PaymentHash doesn't support
+impl PartialOrd for HTLCInfo2 {
+    fn partial_cmp(&self, other: &Self) -> Option<cmp::Ordering> {
+        Some(self.cmp(other))
+    }
+}
+
+// Implement manually so we can have hex encoded payment_hash.
+impl fmt::Debug for HTLCInfo2 {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("HTLCInfo2")
+            .field("value_sat", &self.value_sat)
+            .field("payment_hash", &self.payment_hash.0.to_hex())
+            .field("cltv_expiry", &self.cltv_expiry)
+            .finish()
+    }
+}
+
+#[derive(Debug, Clone, PartialEq)]
+#[allow(missing_docs)]
 pub struct CommitmentInfo2 {
     pub is_counterparty_broadcaster: bool,
     pub to_countersigner_pubkey: PublicKey,
@@ -352,9 +307,50 @@ pub struct CommitmentInfo2 {
     pub offered_htlcs: Vec<HTLCInfo2>,
     pub received_htlcs: Vec<HTLCInfo2>,
 }
-// END NOT TESTED
+
+impl CommitmentInfo2 {
+    /// Construct a normalized CommitmentInfo2
+    pub fn new(
+        is_counterparty_broadcaster: bool,
+        to_countersigner_pubkey: PublicKey,
+        to_countersigner_value_sat: u64,
+        revocation_pubkey: PublicKey,
+        to_broadcaster_delayed_pubkey: PublicKey,
+        to_broadcaster_value_sat: u64,
+        to_self_delay: u16,
+        offered_htlcs: Vec<HTLCInfo2>,
+        received_htlcs: Vec<HTLCInfo2>,
+    ) -> CommitmentInfo2 {
+        CommitmentInfo2 {
+            is_counterparty_broadcaster,
+            to_countersigner_pubkey,
+            to_countersigner_value_sat,
+            revocation_pubkey,
+            to_broadcaster_delayed_pubkey,
+            to_broadcaster_value_sat,
+            to_self_delay,
+            offered_htlcs,
+            received_htlcs,
+        }
+        .normalize()
+    }
+
+    /// Normalize the CommitmentInfo2 for future use
+    pub fn normalize(mut self) -> Self {
+        // Sort the offered and received HTLCs for later comparison
+        self.offered_htlcs.sort();
+        self.received_htlcs.sort();
+        self
+    }
+
+    /// Returns true if there are no pending HTLCS
+    pub fn htlcs_is_empty(&self) -> bool {
+        self.offered_htlcs.is_empty() && self.received_htlcs.is_empty()
+    }
+}
 
 #[allow(dead_code)]
+#[allow(missing_docs)]
 pub struct CommitmentInfo {
     pub is_counterparty_broadcaster: bool,
     pub to_countersigner_address: Option<Payload>,
@@ -372,7 +368,6 @@ pub struct CommitmentInfo {
 }
 
 // Define manually because Payload's fmt::Debug is lame.
-// BEGIN NOT TESTED
 impl fmt::Debug for CommitmentInfo {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("CommitmentInfo")
@@ -413,20 +408,139 @@ impl fmt::Debug for CommitmentInfo {
             .finish()
     }
 }
-// END NOT TESTED
+
+pub(crate) fn parse_received_htlc_script(
+    script: &Script,
+    option_anchor_outputs: bool,
+) -> Result<(Vec<u8>, Vec<u8>, Vec<u8>, Vec<u8>, i64), ValidationError> {
+    let iter = &mut script.instructions();
+    expect_op(iter, OP_DUP)?;
+    expect_op(iter, OP_HASH160)?;
+    let revocation_hash = expect_data(iter)?;
+    expect_op(iter, OP_EQUAL)?;
+    expect_op(iter, OP_IF)?;
+    expect_op(iter, OP_CHECKSIG)?;
+    expect_op(iter, OP_ELSE)?;
+    let remote_htlc_pubkey = expect_data(iter)?;
+    expect_op(iter, OP_SWAP)?;
+    expect_op(iter, OP_SIZE)?;
+    let thirty_two = expect_number(iter)?;
+    if thirty_two != 32 {
+        return Err(mismatch_error(format!("expected 32, saw {}", thirty_two)));
+    }
+    expect_op(iter, OP_EQUAL)?;
+    expect_op(iter, OP_IF)?;
+    expect_op(iter, OP_HASH160)?;
+    let payment_hash_vec = expect_data(iter)?;
+    expect_op(iter, OP_EQUALVERIFY)?;
+    expect_op(iter, OP_PUSHNUM_2)?;
+    expect_op(iter, OP_SWAP)?;
+    let local_htlc_pubkey = expect_data(iter)?;
+    expect_op(iter, OP_PUSHNUM_2)?;
+    expect_op(iter, OP_CHECKMULTISIG)?;
+    expect_op(iter, OP_ELSE)?;
+    expect_op(iter, OP_DROP)?;
+    let cltv_expiry = expect_number(iter)?;
+    expect_op(iter, OP_CLTV)?;
+    expect_op(iter, OP_DROP)?;
+    expect_op(iter, OP_CHECKSIG)?;
+    expect_op(iter, OP_ENDIF)?;
+    if option_anchor_outputs {
+        expect_op(iter, OP_PUSHNUM_1)?;
+        expect_op(iter, OP_CSV)?;
+        expect_op(iter, OP_DROP)?;
+    }
+    expect_op(iter, OP_ENDIF)?;
+    expect_script_end(iter)?;
+    Ok((
+        revocation_hash,
+        remote_htlc_pubkey,
+        payment_hash_vec,
+        local_htlc_pubkey,
+        cltv_expiry,
+    ))
+}
+
+pub(crate) fn parse_offered_htlc_script(
+    script: &Script,
+    option_anchor_outputs: bool,
+) -> Result<(Vec<u8>, Vec<u8>, Vec<u8>, Vec<u8>), ValidationError> {
+    let iter = &mut script.instructions();
+    expect_op(iter, OP_DUP)?;
+    expect_op(iter, OP_HASH160)?;
+    let revocation_hash = expect_data(iter)?;
+    expect_op(iter, OP_EQUAL)?;
+    expect_op(iter, OP_IF)?;
+    expect_op(iter, OP_CHECKSIG)?;
+    expect_op(iter, OP_ELSE)?;
+    let remote_htlc_pubkey = expect_data(iter)?;
+    expect_op(iter, OP_SWAP)?;
+    expect_op(iter, OP_SIZE)?;
+    let thirty_two = expect_number(iter)?;
+    if thirty_two != 32 {
+        return Err(mismatch_error(format!("expected 32, saw {}", thirty_two)));
+    }
+    expect_op(iter, OP_EQUAL)?;
+    expect_op(iter, OP_NOTIF)?;
+    expect_op(iter, OP_DROP)?;
+    expect_op(iter, OP_PUSHNUM_2)?;
+    expect_op(iter, OP_SWAP)?;
+    let local_htlc_pubkey = expect_data(iter)?;
+    expect_op(iter, OP_PUSHNUM_2)?;
+    expect_op(iter, OP_CHECKMULTISIG)?;
+    expect_op(iter, OP_ELSE)?;
+    expect_op(iter, OP_HASH160)?;
+    let payment_hash_vec = expect_data(iter)?;
+    expect_op(iter, OP_EQUALVERIFY)?;
+    expect_op(iter, OP_CHECKSIG)?;
+    expect_op(iter, OP_ENDIF)?;
+    if option_anchor_outputs {
+        expect_op(iter, OP_PUSHNUM_1)?;
+        expect_op(iter, OP_CSV)?;
+        expect_op(iter, OP_DROP)?;
+    }
+    expect_op(iter, OP_ENDIF)?;
+    expect_script_end(iter)?;
+    Ok((
+        revocation_hash,
+        remote_htlc_pubkey,
+        local_htlc_pubkey,
+        payment_hash_vec,
+    ))
+}
+
+pub(crate) fn parse_revokeable_redeemscript(
+    script: &Script,
+    _option_anchor_outputs: bool,
+) -> Result<(Vec<u8>, i64, Vec<u8>), ValidationError> {
+    let iter = &mut script.instructions();
+    expect_op(iter, OP_IF)?;
+    let revocation_key = expect_data(iter)?;
+    expect_op(iter, OP_ELSE)?;
+    let contest_delay = expect_number(iter)?;
+    expect_op(iter, OP_CSV)?;
+    expect_op(iter, OP_DROP)?;
+    let delayed_pubkey = expect_data(iter)?;
+    expect_op(iter, OP_ENDIF)?;
+    expect_op(iter, OP_CHECKSIG)?;
+    expect_script_end(iter)?;
+    Ok((revocation_key, contest_delay, delayed_pubkey))
+}
 
 impl CommitmentInfo {
     // FIXME - should the new_for_{holder,counterparty} wrappers move
     // to Validator::make_info_for_{holder,counterparty}?
-
-    pub fn new_for_holder() -> Self {
+    #[cfg(test)]
+    pub(crate) fn new_for_holder() -> Self {
         CommitmentInfo::new(false)
     }
 
-    pub fn new_for_counterparty() -> Self {
+    #[cfg(test)]
+    pub(crate) fn new_for_counterparty() -> Self {
         CommitmentInfo::new(true)
     }
 
+    /// Construct
     pub fn new(is_counterparty_broadcaster: bool) -> Self {
         CommitmentInfo {
             is_counterparty_broadcaster,
@@ -444,14 +558,15 @@ impl CommitmentInfo {
         }
     }
 
-    pub fn has_to_broadcaster(&self) -> bool {
+    pub(crate) fn has_to_broadcaster(&self) -> bool {
         self.to_broadcaster_delayed_pubkey.is_some()
     }
 
-    pub fn has_to_countersigner(&self) -> bool {
+    pub(crate) fn has_to_countersigner(&self) -> bool {
         self.to_countersigner_address.is_some() || self.to_countersigner_pubkey.is_some()
     }
 
+    /// The amount used by the broadcaster anchor
     pub fn to_broadcaster_anchor_value_sat(&self) -> u64 {
         if self.to_broadcaster_anchor_count == 1 {
             ANCHOR_SAT
@@ -460,6 +575,7 @@ impl CommitmentInfo {
         }
     }
 
+    /// The amount used by the countersigner anchor
     pub fn to_countersigner_anchor_value_sat(&self) -> u64 {
         if self.to_countersigner_anchor_count == 1 {
             ANCHOR_SAT
@@ -492,16 +608,19 @@ impl CommitmentInfo {
         vals: (Vec<u8>, i64, Vec<u8>),
     ) -> Result<(), ValidationError> {
         let (revocation_pubkey, delay, delayed_pubkey) = vals;
-        // policy-v1-commitment-singular-to-local
+        // policy-commitment-singular-to-holder
+        // policy-commitment-singular-to-counterparty
         if self.has_to_broadcaster() {
-            return Err(TransactionFormat("already have to local".to_string()));
+            return Err(transaction_format_error(
+                "more than one to_broadcaster output".to_string(),
+            ));
         }
 
         if delay < 0 {
-            return Err(ScriptFormat("negative delay".to_string())); // NOT TESTED
+            return Err(script_format_error("negative delay".to_string()));
         }
         if delay > MAX_DELAY {
-            return Err(ScriptFormat("delay too large".to_string())); // NOT TESTED
+            return Err(script_format_error("delay too large".to_string()));
         }
 
         // This is safe because we checked for negative
@@ -509,17 +628,16 @@ impl CommitmentInfo {
         self.to_broadcaster_value_sat = out.value;
         self.to_broadcaster_delayed_pubkey = Some(
             PublicKey::from_slice(delayed_pubkey.as_slice())
-                .map_err(|err| Mismatch(format!("delayed_pubkey malformed: {}", err)))?,
-        ); // NOT TESTED
+                .map_err(|err| mismatch_error(format!("delayed_pubkey malformed: {}", err)))?,
+        );
         self.revocation_pubkey = Some(
             PublicKey::from_slice(revocation_pubkey.as_slice())
-                .map_err(|err| Mismatch(format!("revocation_pubkey malformed: {}", err)))?,
-        ); // NOT TESTED
+                .map_err(|err| mismatch_error(format!("revocation_pubkey malformed: {}", err)))?,
+        );
 
         Ok(())
     }
 
-    // BEGIN NOT TESTED
     fn parse_to_countersigner_delayed_script(
         &self,
         script: &Script,
@@ -539,14 +657,17 @@ impl CommitmentInfo {
         out: &TxOut,
         to_countersigner_delayed_pubkey_data: Vec<u8>,
     ) -> Result<(), ValidationError> {
-        // policy-v1-commitment-singular-to-remote
+        // policy-commitment-singular-to-holder
+        // policy-commitment-singular-to-counterparty
         if self.has_to_countersigner() {
-            return Err(TransactionFormat("more than one to remote".to_string()));
+            return Err(transaction_format_error(
+                "more than one to_countersigner output".to_string(),
+            ));
         }
         self.to_countersigner_pubkey = Some(
             PublicKey::from_slice(to_countersigner_delayed_pubkey_data.as_slice()).map_err(
                 |err| {
-                    Mismatch(format!(
+                    mismatch_error(format!(
                         "to_countersigner delayed pubkey malformed: {}",
                         err
                     ))
@@ -556,62 +677,6 @@ impl CommitmentInfo {
         self.to_countersigner_value_sat = out.value;
         Ok(())
     }
-    // END NOT TESTED
-
-    fn parse_received_htlc_script(
-        &self,
-        script: &Script,
-        option_anchor_outputs: bool,
-    ) -> Result<(Vec<u8>, Vec<u8>, Vec<u8>, Vec<u8>, i64), ValidationError> {
-        let iter = &mut script.instructions();
-        expect_op(iter, OP_DUP)?;
-        expect_op(iter, OP_HASH160)?;
-        let revocation_hash = expect_data(iter)?;
-        expect_op(iter, OP_EQUAL)?;
-        expect_op(iter, OP_IF)?;
-        expect_op(iter, OP_CHECKSIG)?;
-        expect_op(iter, OP_ELSE)?;
-        let remote_htlc_pubkey = expect_data(iter)?;
-        expect_op(iter, OP_SWAP)?;
-        expect_op(iter, OP_SIZE)?;
-        let thirty_two = expect_number(iter)?;
-        if thirty_two != 32 {
-            return Err(Mismatch(format!("expected 32, saw {}", thirty_two))); // NOT TESTED
-        }
-        expect_op(iter, OP_EQUAL)?;
-        expect_op(iter, OP_IF)?;
-        expect_op(iter, OP_HASH160)?;
-        let payment_hash_vec = expect_data(iter)?;
-        expect_op(iter, OP_EQUALVERIFY)?;
-        expect_op(iter, OP_PUSHNUM_2)?;
-        expect_op(iter, OP_SWAP)?;
-        let local_htlc_pubkey = expect_data(iter)?;
-        expect_op(iter, OP_PUSHNUM_2)?;
-        expect_op(iter, OP_CHECKMULTISIG)?;
-        expect_op(iter, OP_ELSE)?;
-        expect_op(iter, OP_DROP)?;
-        let cltv_expiry = expect_number(iter)?;
-        expect_op(iter, OP_CLTV)?;
-        expect_op(iter, OP_DROP)?;
-        expect_op(iter, OP_CHECKSIG)?;
-        expect_op(iter, OP_ENDIF)?;
-        if option_anchor_outputs {
-            // BEGIN NOT TESTED
-            expect_op(iter, OP_PUSHNUM_1)?;
-            expect_op(iter, OP_CSV)?;
-            expect_op(iter, OP_DROP)?;
-            // END NOT TESTED
-        }
-        expect_op(iter, OP_ENDIF)?;
-        expect_script_end(iter)?;
-        Ok((
-            revocation_hash,
-            remote_htlc_pubkey,
-            payment_hash_vec,
-            local_htlc_pubkey,
-            cltv_expiry,
-        ))
-    }
 
     fn handle_received_htlc_output(
         &mut self,
@@ -628,10 +693,10 @@ impl CommitmentInfo {
         let payment_hash_hash = payment_hash_vec
             .as_slice()
             .try_into()
-            .map_err(|_| Mismatch("payment hash RIPEMD160 must be length 20".to_string()))?;
+            .map_err(|_| mismatch_error("payment hash RIPEMD160 must be length 20".to_string()))?;
 
         if cltv_expiry < 0 {
-            return Err(ScriptFormat("negative CLTV".to_string())); // NOT TESTED
+            return Err(script_format_error("negative CLTV".to_string()));
         }
 
         let cltv_expiry = cltv_expiry as u32;
@@ -646,57 +711,6 @@ impl CommitmentInfo {
         Ok(())
     }
 
-    fn parse_offered_htlc_script(
-        &self,
-        script: &Script,
-        option_anchor_outputs: bool,
-    ) -> Result<(Vec<u8>, Vec<u8>, Vec<u8>, Vec<u8>), ValidationError> {
-        let iter = &mut script.instructions();
-        expect_op(iter, OP_DUP)?;
-        expect_op(iter, OP_HASH160)?;
-        let revocation_hash = expect_data(iter)?;
-        expect_op(iter, OP_EQUAL)?;
-        expect_op(iter, OP_IF)?;
-        expect_op(iter, OP_CHECKSIG)?;
-        expect_op(iter, OP_ELSE)?;
-        let remote_htlc_pubkey = expect_data(iter)?;
-        expect_op(iter, OP_SWAP)?;
-        expect_op(iter, OP_SIZE)?;
-        let thirty_two = expect_number(iter)?;
-        if thirty_two != 32 {
-            return Err(Mismatch(format!("expected 32, saw {}", thirty_two))); // NOT TESTED
-        }
-        expect_op(iter, OP_EQUAL)?;
-        expect_op(iter, OP_NOTIF)?;
-        expect_op(iter, OP_DROP)?;
-        expect_op(iter, OP_PUSHNUM_2)?;
-        expect_op(iter, OP_SWAP)?;
-        let local_htlc_pubkey = expect_data(iter)?;
-        expect_op(iter, OP_PUSHNUM_2)?;
-        expect_op(iter, OP_CHECKMULTISIG)?;
-        expect_op(iter, OP_ELSE)?;
-        expect_op(iter, OP_HASH160)?;
-        let payment_hash_vec = expect_data(iter)?;
-        expect_op(iter, OP_EQUALVERIFY)?;
-        expect_op(iter, OP_CHECKSIG)?;
-        expect_op(iter, OP_ENDIF)?;
-        if option_anchor_outputs {
-            // BEGIN NOT TESTED
-            expect_op(iter, OP_PUSHNUM_1)?;
-            expect_op(iter, OP_CSV)?;
-            expect_op(iter, OP_DROP)?;
-            // END NOT TESTED
-        }
-        expect_op(iter, OP_ENDIF)?;
-        expect_script_end(iter)?;
-        Ok((
-            revocation_hash,
-            remote_htlc_pubkey,
-            local_htlc_pubkey,
-            payment_hash_vec,
-        ))
-    }
-
     fn handle_offered_htlc_output(
         &mut self,
         out: &TxOut,
@@ -707,7 +721,7 @@ impl CommitmentInfo {
         let payment_hash_hash = payment_hash_vec
             .as_slice()
             .try_into()
-            .map_err(|_| Mismatch("payment hash RIPEMD160 must be length 20".to_string()))?;
+            .map_err(|_| mismatch_error("payment hash RIPEMD160 must be length 20".to_string()))?;
 
         let htlc = HTLCInfo {
             value_sat: out.value,
@@ -723,7 +737,6 @@ impl CommitmentInfo {
         let iter = &mut script.instructions();
         let to_pubkey_data = expect_data(iter)?;
         expect_op(iter, OP_CHECKSIG)?;
-        // BEGIN NOT TESTED
         expect_op(iter, OP_IFDUP)?;
         expect_op(iter, OP_NOTIF)?;
         expect_op(iter, OP_PUSHNUM_16)?;
@@ -731,27 +744,24 @@ impl CommitmentInfo {
         expect_op(iter, OP_ENDIF)?;
         expect_script_end(iter)?;
         Ok(to_pubkey_data)
-        // END NOT TESTED
     }
 
     fn handle_anchor_output(
         &mut self,
-        keys: &EnforcingSigner,
+        keys: &InMemorySigner,
         out: &TxOut,
         to_pubkey_data: Vec<u8>,
     ) -> Result<(), ValidationError> {
         let to_pubkey = PublicKey::from_slice(to_pubkey_data.as_slice())
-            .map_err(|err| Mismatch(format!("anchor to_pubkey malformed: {}", err)))?;
+            .map_err(|err| mismatch_error(format!("anchor to_pubkey malformed: {}", err)))?;
 
         // These are dependent on which side owns this commitment.
         let (to_broadcaster_funding_pubkey, to_countersigner_funding_pubkey) =
             if self.is_counterparty_broadcaster {
-                // BEGIN NOT TESTED
                 (
                     keys.counterparty_pubkeys().funding_pubkey,
                     keys.pubkeys().funding_pubkey,
                 )
-            // END NOT TESTED
             } else {
                 (
                     keys.pubkeys().funding_pubkey,
@@ -759,61 +769,60 @@ impl CommitmentInfo {
                 )
             };
 
-        // policy-v1-commitment-anchor-amount
+        // policy-commitment-anchor-amount
         if out.value != ANCHOR_SAT {
-            return Err(Mismatch(format!("anchor wrong size: {}", out.value)));
+            return Err(mismatch_error(format!("anchor wrong size: {}", out.value)));
         }
 
         if to_pubkey == to_broadcaster_funding_pubkey {
             // local anchor
-            self.to_broadcaster_anchor_count += 1; // NOT TESTED
+            self.to_broadcaster_anchor_count += 1;
         } else if to_pubkey == to_countersigner_funding_pubkey {
             // remote anchor
-            self.to_countersigner_anchor_count += 1; // NOT TESTED
+            self.to_countersigner_anchor_count += 1;
         } else {
-            // policy-v1-commitment-anchor-match-fundingkey
-            return Err(Mismatch(format!(
+            // policy-commitment-anchor-match-fundingkey
+            return Err(mismatch_error(format!(
                 "anchor to_pubkey {} doesn't match local or remote",
-                hex::encode(to_pubkey_data)
+                to_pubkey_data.to_hex()
             )));
         }
-        Ok(()) // NOT TESTED
+        Ok(())
     }
 
-    pub fn handle_output(
+    pub(crate) fn handle_output(
         &mut self,
-        keys: &EnforcingSigner,
+        keys: &InMemorySigner,
         setup: &ChannelSetup,
         out: &TxOut,
         script_bytes: &[u8],
     ) -> Result<(), ValidationError> {
-        // FIXME - This routine is only called on "remote" commitments. Remove this
-        // assert when that is not the case ...
-        assert!(self.is_counterparty_broadcaster);
-
         if out.script_pubkey.is_v0_p2wpkh() {
             // FIXME - Does this need it's own policy tag?
             if setup.option_anchor_outputs() {
-                return Err(TransactionFormat(
+                return Err(transaction_format_error(
                     "p2wpkh to_countersigner not valid with anchors".to_string(),
                 ));
             }
-            // policy-v1-commitment-singular-to-remote
+            // policy-commitment-singular-to-holder
+            // policy-commitment-singular-to-counterparty
             if self.has_to_countersigner() {
-                return Err(TransactionFormat(
-                    "more than one to_countersigner".to_string(),
+                return Err(transaction_format_error(
+                    "more than one to_countersigner output".to_string(),
                 ));
             }
             self.to_countersigner_address = Payload::from_script(&out.script_pubkey);
             self.to_countersigner_value_sat = out.value;
         } else if out.script_pubkey.is_v0_p2wsh() {
             if script_bytes.is_empty() {
-                return Err(TransactionFormat("missing witscript for p2wsh".to_string()));
+                return Err(transaction_format_error(
+                    "missing witscript for p2wsh".to_string(),
+                ));
             }
             let script = Script::from(script_bytes.to_vec());
             // FIXME - Does this need it's own policy tag?
             if out.script_pubkey != script.to_v0_p2wsh() {
-                return Err(TransactionFormat(
+                return Err(transaction_format_error(
                     "script pubkey doesn't match inner script".to_string(),
                 ));
             }
@@ -821,11 +830,11 @@ impl CommitmentInfo {
             if vals.is_ok() {
                 return self.handle_to_broadcaster_output(out, vals.unwrap());
             }
-            let vals = self.parse_received_htlc_script(&script, setup.option_anchor_outputs());
+            let vals = parse_received_htlc_script(&script, setup.option_anchor_outputs());
             if vals.is_ok() {
                 return self.handle_received_htlc_output(out, vals.unwrap());
             }
-            let vals = self.parse_offered_htlc_script(&script, setup.option_anchor_outputs());
+            let vals = parse_offered_htlc_script(&script, setup.option_anchor_outputs());
             if vals.is_ok() {
                 return self.handle_offered_htlc_output(out, vals.unwrap());
             }
@@ -834,18 +843,16 @@ impl CommitmentInfo {
                 return self.handle_anchor_output(keys, out, vals.unwrap());
             }
             if setup.option_anchor_outputs() {
-                // BEGIN NOT TESTED
                 let vals = self.parse_to_countersigner_delayed_script(&script);
                 if vals.is_ok() {
                     return self.handle_to_countersigner_delayed_output(out, vals.unwrap());
                 }
-                // END NOT TESTED
             }
-            // policy-v1-commitment-no-unrecognized-outputs
-            return Err(TransactionFormat("unknown p2wsh script".to_string()));
+            // policy-commitment-no-unrecognized-outputs
+            return Err(transaction_format_error("unknown p2wsh script".to_string()));
         } else {
-            // policy-v1-commitment-no-unrecognized-outputs
-            return Err(TransactionFormat("unknown output type".to_string()));
+            // policy-commitment-no-unrecognized-outputs
+            return Err(transaction_format_error("unknown output type".to_string()));
         }
         Ok(())
     }
@@ -857,13 +864,50 @@ mod tests {
     use bitcoin::secp256k1::{Secp256k1, SecretKey};
     use bitcoin::{Address, Network};
 
-    use crate::node::node::CommitmentType;
-    use crate::util::test_utils::{
-        make_reasonable_test_channel_setup, make_test_channel_keys, make_test_pubkey,
-    };
+    use crate::channel::CommitmentType;
+    use crate::util::key_utils::make_test_pubkey;
+    use crate::util::test_utils::{hex_encode, make_test_channel_keys, make_test_channel_setup};
 
     use super::*;
 
+    use test_env_log::test;
+
+    #[test]
+    fn htlc2_sorting() {
+        // Defined in order ...
+        let htlc0 = HTLCInfo2 {
+            value_sat: 4000,
+            payment_hash: PaymentHash([1; 32]),
+            cltv_expiry: 2 << 16,
+        };
+        let htlc1 = HTLCInfo2 {
+            value_sat: 4000,
+            payment_hash: PaymentHash([1; 32]),
+            cltv_expiry: 3 << 16,
+        };
+        let htlc2 = HTLCInfo2 {
+            value_sat: 4000,
+            payment_hash: PaymentHash([2; 32]),
+            cltv_expiry: 3 << 16,
+        };
+        let htlc3 = HTLCInfo2 {
+            value_sat: 5000,
+            payment_hash: PaymentHash([2; 32]),
+            cltv_expiry: 3 << 16,
+        };
+        let sorted = vec![&htlc0, &htlc1, &htlc2, &htlc3];
+
+        // Reverse order
+        let mut unsorted0 = vec![&htlc3, &htlc2, &htlc1, &htlc0];
+        unsorted0.sort();
+        assert_eq!(unsorted0, sorted);
+
+        // Random order
+        let mut unsorted1 = vec![&htlc2, &htlc0, &htlc3, &htlc1];
+        unsorted1.sort();
+        assert_eq!(unsorted1, sorted);
+    }
+
     #[test]
     fn parse_test_err() {
         let info = CommitmentInfo::new_for_holder();
@@ -899,8 +943,8 @@ mod tests {
         let res = info.handle_to_broadcaster_output(&out, vals.unwrap());
         assert!(res.is_err());
         #[rustfmt::skip]
-        assert_eq!( // NOT TESTED
-            TransactionFormat("already have to local".to_string()),
+        assert_eq!(
+            transaction_format_error("more than one to_broadcaster output".to_string()),
                     res.expect_err("expecting err")
         );
     }
@@ -918,7 +962,7 @@ mod tests {
         assert!(res.is_err());
         assert_eq!(
             res.unwrap_err(),
-            Mismatch(format!("anchor wrong size: {}", out.value))
+            mismatch_error(format!("anchor wrong size: {}", out.value))
         );
     }
 
@@ -935,18 +979,19 @@ mod tests {
         assert!(res.is_err());
         assert_eq!(
             res.unwrap_err(),
-            Mismatch(format!(
+            mismatch_error(format!(
                 "anchor to_pubkey {} doesn\'t match local or remote",
-                hex::encode(to_pubkey_data)
+                hex_encode(&to_pubkey_data)
             ))
         );
     }
 
+    // policy-commitment-no-unrecognized-outputs
     #[test]
     fn handle_output_unknown_output_type_test() {
         let mut info = CommitmentInfo::new_for_counterparty();
         let keys = make_test_channel_keys();
-        let setup = make_reasonable_test_channel_setup();
+        let setup = make_test_channel_setup();
         let out = TxOut {
             value: 42,
             script_pubkey: Default::default(),
@@ -956,15 +1001,16 @@ mod tests {
         assert!(res.is_err());
         assert_eq!(
             res.unwrap_err(),
-            TransactionFormat("unknown output type".to_string())
+            transaction_format_error("unknown output type".to_string())
         );
     }
 
+    // policy-commitment-no-unrecognized-outputs
     #[test]
     fn handle_output_unknown_p2wsh_script_test() {
         let mut info = CommitmentInfo::new_for_counterparty();
         let keys = make_test_channel_keys();
-        let setup = make_reasonable_test_channel_setup();
+        let setup = make_test_channel_setup();
         let script = Builder::new()
             .push_slice(&[0u8; 42]) // invalid
             .into_script();
@@ -976,7 +1022,7 @@ mod tests {
         assert!(res.is_err());
         assert_eq!(
             res.unwrap_err(),
-            TransactionFormat("unknown p2wsh script".to_string())
+            transaction_format_error("unknown p2wsh script".to_string())
         );
     }
 
@@ -984,7 +1030,7 @@ mod tests {
     fn handle_output_p2wpkh_to_countersigner_with_anchors_test() {
         let mut info = CommitmentInfo::new_for_counterparty();
         let keys = make_test_channel_keys();
-        let mut setup = make_reasonable_test_channel_setup();
+        let mut setup = make_test_channel_setup();
         setup.commitment_type = CommitmentType::Anchors;
         let pubkey = bitcoin::PublicKey::from_slice(&make_test_pubkey(43).serialize()[..]).unwrap();
         let out = TxOut {
@@ -997,7 +1043,7 @@ mod tests {
         assert!(res.is_err());
         assert_eq!(
             res.unwrap_err(),
-            TransactionFormat("p2wpkh to_countersigner not valid with anchors".to_string())
+            transaction_format_error("p2wpkh to_countersigner not valid with anchors".to_string())
         );
     }
 
@@ -1005,7 +1051,7 @@ mod tests {
     fn handle_output_more_than_one_to_countersigner_test() {
         let mut info = CommitmentInfo::new_for_counterparty();
         let keys = make_test_channel_keys();
-        let setup = make_reasonable_test_channel_setup();
+        let setup = make_test_channel_setup();
         let pubkey = bitcoin::PublicKey::from_slice(&make_test_pubkey(43).serialize()[..]).unwrap();
         let address = Address::p2wpkh(&pubkey, Network::Testnet).unwrap();
         let out = TxOut {
@@ -1020,7 +1066,7 @@ mod tests {
         assert!(res.is_err());
         assert_eq!(
             res.unwrap_err(),
-            TransactionFormat("more than one to_countersigner".to_string())
+            transaction_format_error("more than one to_countersigner output".to_string())
         );
     }
 
@@ -1028,7 +1074,7 @@ mod tests {
     fn handle_output_missing_witscript_test() {
         let mut info = CommitmentInfo::new_for_counterparty();
         let keys = make_test_channel_keys();
-        let setup = make_reasonable_test_channel_setup();
+        let setup = make_test_channel_setup();
         let script = Builder::new().into_script();
         let out = TxOut {
             value: 42,
@@ -1038,7 +1084,7 @@ mod tests {
         assert!(res.is_err());
         assert_eq!(
             res.unwrap_err(),
-            TransactionFormat("missing witscript for p2wsh".to_string())
+            transaction_format_error("missing witscript for p2wsh".to_string())
         );
     }
 
@@ -1046,7 +1092,7 @@ mod tests {
     fn handle_output_script_pubkey_doesnt_match_test() {
         let mut info = CommitmentInfo::new_for_counterparty();
         let keys = make_test_channel_keys();
-        let setup = make_reasonable_test_channel_setup();
+        let setup = make_test_channel_setup();
         let script0 = Builder::new().into_script();
         let script1 = Builder::new().push_slice(&[0u8; 42]).into_script();
         let out = TxOut {
@@ -1057,7 +1103,7 @@ mod tests {
         assert!(res.is_err());
         assert_eq!(
             res.unwrap_err(),
-            TransactionFormat("script pubkey doesn\'t match inner script".to_string())
+            transaction_format_error("script pubkey doesn\'t match inner script".to_string())
         );
     }
 }
