@@ -1,8 +1,10 @@
+#![allow(missing_docs)]
 //! A bunch of useful utilities for building networks of nodes and exchanging messages between
 //! nodes for functional tests.
 
 use core::cell::RefCell;
-use crate::{Rc, Mutex};
+use crate::Rc;
+use crate::sync::Mutex;
 
 use bitcoin;
 use bitcoin::{Block, Network, Transaction, TxOut};
@@ -16,7 +18,8 @@ use chain::transaction::OutPoint;
 use lightning::chain;
 use lightning::chain::{Confirm, Listen, chaininterface};
 use lightning::ln;
-use lightning::ln::channelmanager::{BestBlock, ChainParameters};
+use lightning::ln::channelmanager::ChainParameters;
+use lightning::chain::BestBlock;
 use lightning::ln::features::InvoiceFeatures;
 use lightning::ln::functional_test_utils::ConnectStyle;
 use lightning::ln::PaymentSecret;
@@ -30,11 +33,13 @@ use ln::channelmanager::ChannelManager;
 use ln::features::InitFeatures;
 use ln::msgs;
 use ln::msgs::{ChannelMessageHandler, RoutingMessageHandler};
-use util::events::{Event, EventsProvider, MessageSendEvent, MessageSendEventsProvider};
+use util::events::{Event, MessageSendEvent, MessageSendEventsProvider};
 
 use crate::util::loopback::{LoopbackChannelSigner, LoopbackSignerKeysInterface};
 use crate::util::test_utils::{TestChainMonitor, TestPersister};
 use core::cmp;
+use lightning::util::events::PaymentPurpose;
+use crate::lightning::routing::network_graph::NetworkGraph;
 
 pub const CHAN_CONFIRM_DEPTH: u32 = 10;
 
@@ -118,13 +123,11 @@ fn do_connect_block<'a, 'b, 'c, 'd>(node: &'a Node<'b, 'c, 'd>, block: &Block, s
         }
     }
 
-    // We really want here test_process_background_events, but it's crate-private
-    node.node.timer_tick_occurred();
+    node.node.test_process_background_events();
     node.blocks.borrow_mut().push((block.header, height));
 }
 
 
-// BEGIN NOT TESTED
 pub fn disconnect_block<'a, 'b, 'c, 'd>(
     node: &'a Node<'b, 'c, 'd>,
     header: &BlockHeader,
@@ -135,7 +138,6 @@ pub fn disconnect_block<'a, 'b, 'c, 'd>(
         .block_disconnected(header, height);
     node.node.block_disconnected(header, height);
 }
-// END NOT TESTED
 
 pub struct TestChanMonCfg {
     pub tx_broadcaster: TestBroadcaster,
@@ -197,7 +199,6 @@ impl<'a, 'b, 'c> Drop for Node<'a, 'b, 'c> {
     }
 }
 
-// BEGIN NOT TESTED
 pub fn create_chan_between_nodes<'a, 'b, 'c, 'd>(
     node_a: &'a Node<'b, 'c, 'd>,
     node_b: &'a Node<'b, 'c, 'd>,
@@ -212,7 +213,6 @@ pub fn create_chan_between_nodes<'a, 'b, 'c, 'd>(
 ) {
     create_chan_between_nodes_with_value(node_a, node_b, 100000, 10001, a_flags, b_flags)
 }
-// END NOT TESTED
 
 pub fn create_chan_between_nodes_with_value<'a, 'b, 'c, 'd>(
     node_a: &'a Node<'b, 'c, 'd>,
@@ -292,6 +292,25 @@ macro_rules! get_event_msg {
     }};
 }
 
+///
+#[macro_export]
+macro_rules! get_htlc_update_msgs {
+	($node: expr, $node_id: expr) => {
+		{
+			let events = $node.node.get_and_clear_pending_msg_events();
+			assert_eq!(events.len(), 1);
+			match events[0] {
+				MessageSendEvent::UpdateHTLCs { ref node_id, ref updates } => {
+					assert_eq!(*node_id, $node_id);
+					(*updates).clone()
+				},
+				_ => panic!("Unexpected event"),
+			}
+		}
+	}
+}
+
+///
 #[macro_export]
 macro_rules! get_local_commitment_txn {
     ($node: expr, $channel_id: expr) => {{
@@ -336,6 +355,7 @@ macro_rules! check_closed_broadcast {
 	}}
 }
 
+///
 #[macro_export]
 macro_rules! check_added_monitors {
     ($node: expr, $count: expr) => {{
@@ -486,8 +506,22 @@ pub fn create_announced_chan_between_nodes_with_value<'a, 'b, 'c, 'd>(nodes: &'a
 pub fn update_nodes_with_chan_announce<'a, 'b, 'c, 'd>(nodes: &'a Vec<Node<'b, 'c, 'd>>, a: usize, b: usize, ann: &msgs::ChannelAnnouncement, upd_1: &msgs::ChannelUpdate, upd_2: &msgs::ChannelUpdate) {
     nodes[a].node.broadcast_node_announcement([0, 0, 0], [0; 32], Vec::new());
     let a_events = nodes[a].node.get_and_clear_pending_msg_events();
-    assert_eq!(a_events.len(), 1);
-    let a_node_announcement = match a_events[0] {
+    assert!(a_events.len() >= 2);
+
+    // ann should be re-generated by broadcast_node_announcement - check that we have it.
+    let mut found_ann_1 = false;
+    for event in a_events.iter() {
+        match event {
+            MessageSendEvent::BroadcastChannelAnnouncement { ref msg, .. } => {
+                if msg == ann { found_ann_1 = true; }
+            },
+            MessageSendEvent::BroadcastNodeAnnouncement { .. } => {},
+            _ => panic!("Unexpected event {:?}", event),
+        }
+    }
+    assert!(found_ann_1);
+
+    let a_node_announcement = match a_events.last().unwrap() {
         MessageSendEvent::BroadcastNodeAnnouncement { ref msg } => {
             (*msg).clone()
         },
@@ -496,8 +530,22 @@ pub fn update_nodes_with_chan_announce<'a, 'b, 'c, 'd>(nodes: &'a Vec<Node<'b, '
 
     nodes[b].node.broadcast_node_announcement([1, 1, 1], [1; 32], Vec::new());
     let b_events = nodes[b].node.get_and_clear_pending_msg_events();
-    assert_eq!(b_events.len(), 1);
-    let b_node_announcement = match b_events[0] {
+    assert!(b_events.len() >= 2);
+
+    // ann should be re-generated by broadcast_node_announcement - check that we have it.
+    let mut found_ann_2 = false;
+    for event in b_events.iter() {
+        match event {
+            MessageSendEvent::BroadcastChannelAnnouncement { ref msg, .. } => {
+                if msg == ann { found_ann_2 = true; }
+            },
+            MessageSendEvent::BroadcastNodeAnnouncement { .. } => {},
+            _ => panic!("Unexpected event"),
+        }
+    }
+    assert!(found_ann_2);
+
+    let b_node_announcement = match b_events.last().unwrap() {
         MessageSendEvent::BroadcastNodeAnnouncement { ref msg } => {
             (*msg).clone()
         },
@@ -513,18 +561,33 @@ pub fn update_nodes_with_chan_announce<'a, 'b, 'c, 'd>(nodes: &'a Vec<Node<'b, '
     }
 }
 
+///
 #[macro_export]
 macro_rules! check_spends {
-    ($tx: expr, $spends_tx: expr) => {{
-        $tx.verify(|out_point| {
-            if out_point.txid == $spends_tx.txid() {
-                $spends_tx.output.get(out_point.vout as usize).cloned()
-            } else {
-                None // NOT TESTED
-            }
-        })
-        .unwrap();
-    }};
+	($tx: expr, $($spends_txn: expr),*) => {
+		{
+			let get_output = |out_point: &bitcoin::blockdata::transaction::OutPoint| {
+				$(
+					if out_point.txid == $spends_txn.txid() {
+						return $spends_txn.output.get(out_point.vout as usize).cloned()
+					}
+				)*
+				None
+			};
+			let mut total_value_in = 0;
+			for input in $tx.input.iter() {
+				total_value_in += get_output(&input.previous_output).unwrap().value;
+			}
+			let mut total_value_out = 0;
+			for output in $tx.output.iter() {
+				total_value_out += output.value;
+			}
+			let min_fee = ($tx.get_weight() as u64 + 3) / 4; // One sat per vbyte (ie per weight/4, rounded up)
+			// Input amount - output amount = fee, so check that out + min_fee is smaller than input
+			assert!(total_value_out + min_fee <= total_value_in);
+			$tx.verify(get_output).unwrap();
+		}
+	}
 }
 
 macro_rules! get_closing_signed_broadcast {
@@ -554,72 +617,52 @@ macro_rules! get_closing_signed_broadcast {
     }};
 }
 
-pub fn close_channel<'a, 'b, 'c>(
-    outbound_node: &Node<'a, 'b, 'c>,
-    inbound_node: &Node<'a, 'b, 'c>,
-    channel_id: &[u8; 32],
-    funding_tx: Transaction,
-    close_inbound_first: bool,
-) -> (msgs::ChannelUpdate, msgs::ChannelUpdate, Transaction) {
-    let (node_a, broadcaster_a, struct_a) = if close_inbound_first {
-        (
-            &inbound_node.node,
-            &inbound_node.tx_broadcaster,
-            inbound_node,
-        )
-    } else {
-        // BEGIN NOT TESTED
-        (
-            &outbound_node.node,
-            &outbound_node.tx_broadcaster,
-            outbound_node,
-        )
-        // END NOT TESTED
-    };
-    let (node_b, broadcaster_b) = if close_inbound_first {
-        (&outbound_node.node, &outbound_node.tx_broadcaster)
-    } else {
-        (&inbound_node.node, &inbound_node.tx_broadcaster) // NOT TESTED
-    };
+///
+#[macro_export]
+macro_rules! check_closed_event {
+       ($node: expr, $events: expr, $reason: expr) => {{
+               let events = $node.node.get_and_clear_pending_events();
+               assert_eq!(events.len(), $events);
+               let expected_reason = $reason;
+               for event in events {
+                       match event {
+                               Event::ChannelClosed { ref reason, .. } => {
+                                       assert_eq!(*reason, expected_reason);
+                               },
+                               _ => panic!("Unexpected event"),
+                       }
+               }
+       }}
+}
+
+pub fn close_channel<'a, 'b, 'c>(outbound_node: &Node<'a, 'b, 'c>, inbound_node: &Node<'a, 'b, 'c>, channel_id: &[u8; 32], funding_tx: Transaction, close_inbound_first: bool) -> (msgs::ChannelUpdate, msgs::ChannelUpdate, Transaction) {
+    let (node_a, broadcaster_a, struct_a) = if close_inbound_first { (&inbound_node.node, &inbound_node.tx_broadcaster, inbound_node) } else { (&outbound_node.node, &outbound_node.tx_broadcaster, outbound_node) };
+    let (node_b, broadcaster_b, struct_b) = if close_inbound_first { (&outbound_node.node, &outbound_node.tx_broadcaster, outbound_node) } else { (&inbound_node.node, &inbound_node.tx_broadcaster, inbound_node) };
     let (tx_a, tx_b);
 
     node_a.close_channel(channel_id).unwrap();
-    node_b.handle_shutdown(
-        &node_a.get_our_node_id(),
-        &InitFeatures::known(),
-        &get_event_msg!(
-            struct_a,
-            MessageSendEvent::SendShutdown,
-            node_b.get_our_node_id()
-        ),
-    );
+    node_b.handle_shutdown(&node_a.get_our_node_id(), &InitFeatures::known(), &get_event_msg!(struct_a, MessageSendEvent::SendShutdown, node_b.get_our_node_id()));
 
     let events_1 = node_b.get_and_clear_pending_msg_events();
     assert!(events_1.len() >= 1);
     let shutdown_b = match events_1[0] {
-        MessageSendEvent::SendShutdown {
-            ref node_id,
-            ref msg,
-        } => {
+        MessageSendEvent::SendShutdown { ref node_id, ref msg } => {
             assert_eq!(node_id, &node_a.get_our_node_id());
             msg.clone()
-        }
-        _ => panic!("Unexpected event"), // NOT TESTED
+        },
+        _ => panic!("Unexpected event"),
     };
 
     let closing_signed_b = if !close_inbound_first {
-        assert_eq!(events_1.len(), 1); // NOT TESTED
-        None // NOT TESTED
+        assert_eq!(events_1.len(), 1);
+        None
     } else {
         Some(match events_1[1] {
-            MessageSendEvent::SendClosingSigned {
-                ref node_id,
-                ref msg,
-            } => {
+            MessageSendEvent::SendClosingSigned { ref node_id, ref msg } => {
                 assert_eq!(node_id, &node_a.get_our_node_id());
                 msg.clone()
-            }
-            _ => panic!("Unexpected event"), // NOT TESTED
+            },
+            _ => panic!("Unexpected event"),
         })
     };
 
@@ -627,30 +670,11 @@ pub fn close_channel<'a, 'b, 'c>(
     let (as_update, bs_update) = if close_inbound_first {
         assert!(node_a.get_and_clear_pending_msg_events().is_empty());
         node_a.handle_closing_signed(&node_b.get_our_node_id(), &closing_signed_b.unwrap());
-        assert_eq!(broadcaster_a.txn_broadcasted.lock().unwrap().len(), 1);
-        tx_a = broadcaster_a.txn_broadcasted.lock().unwrap().remove(0);
-        let (as_update, closing_signed_a) =
-            get_closing_signed_broadcast!(node_a, node_b.get_our_node_id());
 
-        node_b.handle_closing_signed(&node_a.get_our_node_id(), &closing_signed_a.unwrap());
-        let (bs_update, none_b) = get_closing_signed_broadcast!(node_b, node_a.get_our_node_id());
-        assert!(none_b.is_none());
+        node_b.handle_closing_signed(&node_a.get_our_node_id(), &get_event_msg!(struct_a, MessageSendEvent::SendClosingSigned, node_b.get_our_node_id()));
         assert_eq!(broadcaster_b.txn_broadcasted.lock().unwrap().len(), 1);
         tx_b = broadcaster_b.txn_broadcasted.lock().unwrap().remove(0);
-        (as_update, bs_update)
-    } else {
-        // BEGIN NOT TESTED
-        let closing_signed_a = get_event_msg!(
-            struct_a,
-            MessageSendEvent::SendClosingSigned,
-            node_b.get_our_node_id()
-        );
-
-        node_b.handle_closing_signed(&node_a.get_our_node_id(), &closing_signed_a);
-        assert_eq!(broadcaster_b.txn_broadcasted.lock().unwrap().len(), 1);
-        tx_b = broadcaster_b.txn_broadcasted.lock().unwrap().remove(0);
-        let (bs_update, closing_signed_b) =
-            get_closing_signed_broadcast!(node_b, node_a.get_our_node_id());
+        let (bs_update, closing_signed_b) = get_closing_signed_broadcast!(node_b, node_a.get_our_node_id());
 
         node_a.handle_closing_signed(&node_b.get_our_node_id(), &closing_signed_b.unwrap());
         let (as_update, none_a) = get_closing_signed_broadcast!(node_a, node_b.get_our_node_id());
@@ -658,7 +682,22 @@ pub fn close_channel<'a, 'b, 'c>(
         assert_eq!(broadcaster_a.txn_broadcasted.lock().unwrap().len(), 1);
         tx_a = broadcaster_a.txn_broadcasted.lock().unwrap().remove(0);
         (as_update, bs_update)
-        // END NOT TESTED
+    } else {
+        let closing_signed_a = get_event_msg!(struct_a, MessageSendEvent::SendClosingSigned, node_b.get_our_node_id());
+
+        node_b.handle_closing_signed(&node_a.get_our_node_id(), &closing_signed_a);
+        node_a.handle_closing_signed(&node_b.get_our_node_id(), &get_event_msg!(struct_b, MessageSendEvent::SendClosingSigned, node_a.get_our_node_id()));
+
+        assert_eq!(broadcaster_a.txn_broadcasted.lock().unwrap().len(), 1);
+        tx_a = broadcaster_a.txn_broadcasted.lock().unwrap().remove(0);
+        let (as_update, closing_signed_a) = get_closing_signed_broadcast!(node_a, node_b.get_our_node_id());
+
+        node_b.handle_closing_signed(&node_a.get_our_node_id(), &closing_signed_a.unwrap());
+        let (bs_update, none_b) = get_closing_signed_broadcast!(node_b, node_a.get_our_node_id());
+        assert!(none_b.is_none());
+        assert_eq!(broadcaster_b.txn_broadcasted.lock().unwrap().len(), 1);
+        tx_b = broadcaster_b.txn_broadcasted.lock().unwrap().remove(0);
+        (as_update, bs_update)
     };
     assert_eq!(tx_a, tx_b);
     check_spends!(tx_a, funding_tx);
@@ -693,17 +732,15 @@ impl SendEvent {
             MessageSendEvent::UpdateHTLCs { node_id, updates } => {
                 SendEvent::from_commitment_update(node_id, updates)
             }
-            _ => panic!("Unexpected event type!"), // NOT TESTED
+            _ => panic!("Unexpected event type!"),
         }
     }
 
-    // BEGIN NOT TESTED
     pub fn from_node<'a, 'b, 'c>(node: &Node<'a, 'b, 'c>) -> SendEvent {
         let mut events = node.node.get_and_clear_pending_msg_events();
         assert_eq!(events.len(), 1);
         SendEvent::from_event(events.pop().unwrap())
     }
-    // END NOT TESTED
 }
 
 macro_rules! commitment_signed_dance {
@@ -743,7 +780,7 @@ macro_rules! commitment_signed_dance {
                     }
                     _ => panic!("Unexpected event"),
                 },
-                events.get(1).map(|e| e.clone()), // NOT TESTED
+                events.get(1).map(|e| e.clone()),
             )
         };
         check_added_monitors!($node_b, 1);
@@ -815,6 +852,21 @@ macro_rules! expect_pending_htlcs_forwardable {
     }};
 }
 
+///
+#[macro_export]
+macro_rules! expect_pending_htlcs_forwardable_from_events {
+	($node: expr, $events: expr, $ignore: expr) => {{
+		assert_eq!($events.len(), 1);
+		match $events[0] {
+			Event::PendingHTLCsForwardable { .. } => { },
+			_ => panic!("Unexpected event"),
+		};
+		if $ignore {
+			$node.node.process_pending_htlc_forwards();
+		}
+	}}
+}
+
 macro_rules! expect_payment_sent {
     ($node: expr, $expected_payment_preimage: expr) => {
         let events = $node.node.get_and_clear_pending_events();
@@ -830,6 +882,7 @@ macro_rules! expect_payment_sent {
     };
 }
 
+///
 #[macro_export]
 macro_rules! expect_pending_htlcs_forwardable_ignore {
 	($node: expr) => {{
@@ -844,6 +897,23 @@ macro_rules! expect_pending_htlcs_forwardable_ignore {
 	}}
 }
 
+///
+#[macro_export]
+macro_rules! expect_payment_forwarded {
+	($node: expr, $expected_fee: expr, $upstream_force_closed: expr) => {
+		let events = $node.node.get_and_clear_pending_events();
+		assert_eq!(events.len(), 1);
+		match events[0] {
+			Event::PaymentForwarded { fee_earned_msat, claim_from_onchain_tx } => {
+				assert_eq!(fee_earned_msat, $expected_fee);
+				assert_eq!(claim_from_onchain_tx, $upstream_force_closed);
+			},
+			_ => panic!("Unexpected event"),
+		}
+	}
+}
+
+///
 #[macro_export]
 macro_rules! expect_payment_failed {
 	($node: expr, $expected_payment_hash: expr, $rejected_by_dest: expr $(, $expected_error_code: expr, $expected_error_data: expr)*) => {
@@ -852,7 +922,7 @@ macro_rules! expect_payment_failed {
 		let events = $node.node.get_and_clear_pending_events();
 		assert_eq!(events.len(), 1);
 		match events[0] {
-			Event::PaymentFailed { ref payment_hash, rejected_by_dest, /*ref error_code, ref error_data*/ } => {
+			Event::PaymentPathFailed { ref payment_hash, rejected_by_dest, .. } => {
 				assert_eq!(*payment_hash, $expected_payment_hash, "unexpected payment_hash");
 				assert_eq!(rejected_by_dest, $rejected_by_dest, "unexpected rejected_by_dest value");
 				// assert!(error_code.is_some(), "expected error_code.is_some() = true");
@@ -873,23 +943,14 @@ pub fn send_along_route_with_secret<'a, 'b, 'c>(origin_node: &Node<'a, 'b, 'c>,
     pass_along_route(origin_node, expected_paths, recv_value, our_payment_hash, our_payment_secret);
 }
 
-pub fn pass_along_path<'a, 'b, 'c>(
-    origin_node: &Node<'a, 'b, 'c>,
-    expected_path: &[&Node<'a, 'b, 'c>],
-    recv_value: u64,
-    our_payment_hash: PaymentHash,
-    our_payment_secret: PaymentSecret,
-    ev: MessageSendEvent,
-    payment_received_expected: bool,
-) {
+pub fn pass_along_path<'a, 'b, 'c>(origin_node: &Node<'a, 'b, 'c>, expected_path: &[&Node<'a, 'b, 'c>], recv_value: u64, our_payment_hash: PaymentHash, our_payment_secret: Option<PaymentSecret>, ev: MessageSendEvent, payment_received_expected: bool, expected_preimage: Option<PaymentPreimage>) {
     let mut payment_event = SendEvent::from_event(ev);
     let mut prev_node = origin_node;
 
     for (idx, &node) in expected_path.iter().enumerate() {
         assert_eq!(node.node.get_our_node_id(), payment_event.node_id);
 
-        node.node
-            .handle_update_add_htlc(&prev_node.node.get_our_node_id(), &payment_event.msgs[0]);
+        node.node.handle_update_add_htlc(&prev_node.node.get_our_node_id(), &payment_event.msgs[0]);
         check_added_monitors!(node, 0);
         commitment_signed_dance!(node, prev_node, payment_event.commitment_msg, false);
 
@@ -900,16 +961,24 @@ pub fn pass_along_path<'a, 'b, 'c>(
             if payment_received_expected {
                 assert_eq!(events_2.len(), 1);
                 match events_2[0] {
-                    Event::PaymentReceived { ref payment_hash, ref payment_preimage, ref payment_secret, amt, user_payment_id: _ } => {
+                    Event::PaymentReceived { ref payment_hash, ref purpose, amt} => {
                         assert_eq!(our_payment_hash, *payment_hash);
-                        assert!(payment_preimage.is_none());
-                        assert_eq!(our_payment_secret, *payment_secret);
+                        match &purpose {
+                            PaymentPurpose::InvoicePayment { payment_preimage, payment_secret, .. } => {
+                                assert_eq!(expected_preimage, *payment_preimage);
+                                assert_eq!(our_payment_secret.unwrap(), *payment_secret);
+                            },
+                            PaymentPurpose::SpontaneousPayment(payment_preimage) => {
+                                assert_eq!(expected_preimage.unwrap(), *payment_preimage);
+                                assert!(our_payment_secret.is_none());
+                            },
+                        }
                         assert_eq!(amt, recv_value);
-                    }
-                    _ => panic!("Unexpected event"), // NOT TESTED
+                    },
+                    _ => panic!("Unexpected event"),
                 }
             } else {
-                assert!(events_2.is_empty()); // NOT TESTED
+                assert!(events_2.is_empty());
             }
         } else {
             let mut events_2 = node.node.get_and_clear_pending_msg_events();
@@ -923,28 +992,14 @@ pub fn pass_along_path<'a, 'b, 'c>(
     }
 }
 
-pub fn pass_along_route<'a, 'b, 'c>(
-    origin_node: &Node<'a, 'b, 'c>,
-    expected_route: &[&[&Node<'a, 'b, 'c>]],
-    recv_value: u64,
-    our_payment_hash: PaymentHash,
-    our_payment_secret: PaymentSecret,
-) {
+pub fn pass_along_route<'a, 'b, 'c>(origin_node: &Node<'a, 'b, 'c>, expected_route: &[&[&Node<'a, 'b, 'c>]], recv_value: u64, our_payment_hash: PaymentHash, our_payment_secret: PaymentSecret) {
     let mut events = origin_node.node.get_and_clear_pending_msg_events();
     assert_eq!(events.len(), expected_route.len());
     for (path_idx, (ev, expected_path)) in events.drain(..).zip(expected_route.iter()).enumerate() {
         // Once we've gotten through all the HTLCs, the last one should result in a
         // PaymentReceived (but each previous one should not!), .
         let expect_payment = path_idx == expected_route.len() - 1;
-        pass_along_path(
-            origin_node,
-            expected_path,
-            recv_value,
-            our_payment_hash.clone(),
-            our_payment_secret,
-            ev,
-            expect_payment,
-        );
+        pass_along_path(origin_node, expected_path, recv_value, our_payment_hash.clone(), Some(our_payment_secret), ev, expect_payment, None);
     }
 }
 
@@ -1001,6 +1056,8 @@ pub fn claim_payment_along_route<'a, 'b, 'c>(origin_node: &Node<'a, 'b, 'c>, exp
 			($node: expr, $prev_node: expr, $new_msgs: expr) => {
 				{
 					$node.node.handle_update_fulfill_htlc(&$prev_node.node.get_our_node_id(), &next_msgs.as_ref().unwrap().0);
+            		let events = $node.node.get_and_clear_pending_events();
+            		assert_eq!(events.len(), 1);
 					check_added_monitors!($node, 1);
 					let new_next_msgs = if $new_msgs {
 						let events = $node.node.get_and_clear_pending_msg_events();
@@ -1046,12 +1103,12 @@ pub fn claim_payment<'a, 'b, 'c>(origin_node: &Node<'a, 'b, 'c>, expected_route:
     claim_payment_along_route(origin_node, &[expected_route], false, our_payment_preimage);
 }
 
-pub const TEST_FINAL_CLTV: u32 = 32;
+pub const TEST_FINAL_CLTV: u32 = 70;
 
 pub fn route_payment<'a, 'b, 'c>(origin_node: &Node<'a, 'b, 'c>, expected_route: &[&Node<'a, 'b, 'c>], recv_value: u64) -> (PaymentPreimage, PaymentHash, PaymentSecret) {
     let net_graph_msg_handler = &origin_node.net_graph_msg_handler;
     let logger = test_utils::TestLogger::new();
-    let route = get_route(&origin_node.node.get_our_node_id(), &net_graph_msg_handler.network_graph.read().unwrap(), &expected_route.last().unwrap().node.get_our_node_id(), Some(InvoiceFeatures::known()), None, &Vec::new(), recv_value, TEST_FINAL_CLTV, &logger).unwrap();
+    let route = get_route(&origin_node.node.get_our_node_id(), &net_graph_msg_handler.network_graph, &expected_route.last().unwrap().node.get_our_node_id(), Some(InvoiceFeatures::known()), None, &Vec::new(), recv_value, TEST_FINAL_CLTV, &logger).unwrap();
     assert_eq!(route.paths.len(), 1);
     assert_eq!(route.paths[0].len(), expected_route.len());
     for (node, hop) in expected_route.iter().zip(route.paths[0].iter()) {
@@ -1081,7 +1138,7 @@ pub fn create_chanmon_cfgs(node_count: usize) -> Vec<TestChanMonCfg> {
         let tx_broadcaster = TestBroadcaster {
             txn_broadcasted: Mutex::new(Vec::new()),
         };
-        let fee_estimator = test_utils::TestFeeEstimator { sat_per_kw: 253 };
+        let fee_estimator = test_utils::TestFeeEstimator { sat_per_kw: Mutex::new(253) };
         let chain_source = test_utils::TestChainSource::new(Network::Testnet);
         let logger = test_utils::TestLogger::with_id(format!("node {}", i));
         let persister = TestPersister::new();
@@ -1150,8 +1207,9 @@ pub fn create_network<'a, 'b: 'a, 'c: 'b>(
     let payment_count = Rc::new(RefCell::new(0));
 
     for i in 0..node_count {
+        let network_graph = NetworkGraph::new(cfgs[i].chain_source.genesis_hash);
         let net_graph_msg_handler =
-            NetGraphMsgHandler::new(cfgs[i].chain_source.genesis_hash, None, cfgs[i].logger);
+            NetGraphMsgHandler::new(network_graph,None, cfgs[i].logger);
         let connect_style = Rc::new(RefCell::new(ConnectStyle::FullBlockViaListen));
         nodes.push(Node {
             chain_source: cfgs[i].chain_source,
@@ -1169,10 +1227,16 @@ pub fn create_network<'a, 'b: 'a, 'c: 'b>(
         })
     }
 
+    for i in 0..node_count {
+        for j in (i+1)..node_count {
+            nodes[i].node.peer_connected(&nodes[j].node.get_our_node_id(), &msgs::Init { features: InitFeatures::known() });
+            nodes[j].node.peer_connected(&nodes[i].node.get_our_node_id(), &msgs::Init { features: InitFeatures::known() });
+        }
+    }
+
     nodes
 }
 
-// BEGIN NOT TESTED
 pub fn dump_node_txn(prefix: &str, node: &Node) {
     let node_txn = node.tx_broadcaster.txn_broadcasted.lock().unwrap();
     dump_txn(prefix, &*node_txn);
@@ -1184,7 +1248,6 @@ pub fn dump_txn(prefix: &str, txn: &Vec<Transaction>) {
         println!("{} {} {:?}", prefix, x.txid(), x);
     }
 }
-// END NOT TESTED
 
 pub fn get_announce_close_broadcast_events<'a, 'b, 'c>(nodes: &Vec<Node<'a, 'b, 'c>>, a: usize, b: usize)  {
     let events_1 = nodes[a].node.get_and_clear_pending_msg_events();
