@@ -1,8 +1,10 @@
-use crate::Arc;
+#![allow(missing_docs)]
+use std::convert::TryInto;
 
+use bitcoin::secp256k1::recovery::RecoverableSignature;
 use bitcoin::secp256k1::{All, PublicKey, Secp256k1, SecretKey, Signature};
-use bitcoin::{Script, Transaction};
-use lightning::chain::keysinterface::{BaseSign, KeysInterface, KeysManager, Sign};
+use bitcoin::{Script, Transaction, TxOut};
+use lightning::chain::keysinterface::{BaseSign, KeysInterface, Sign, SpendableOutputDescriptor};
 use lightning::ln::chan_utils;
 use lightning::ln::chan_utils::{
     ChannelPublicKeys, ChannelTransactionParameters, CommitmentTransaction, HTLCOutputInCommitment,
@@ -10,31 +12,52 @@ use lightning::ln::chan_utils::{
 };
 use lightning::ln::msgs::{DecodeError, UnsignedChannelAnnouncement};
 use lightning::util::ser::{Writeable, Writer};
+use log::{debug, error, info};
 
-use crate::node::node::{ChannelId, ChannelSetup, CommitmentType, Node, ChannelBase};
-use crate::signer::my_signer::MultiSigner;
+use crate::channel::{ChannelBase, ChannelId, ChannelSetup, CommitmentType};
+use crate::io_extras::Error as IOError;
+use crate::node::Node;
+use crate::signer::multi_signer::MultiSigner;
 use crate::tx::tx::HTLCInfo2;
-use crate::util::crypto_utils::{derive_public_key, derive_revocation_pubkey, payload_for_p2wpkh, signature_to_bitcoin_vec};
+use crate::util::crypto_utils::{
+    derive_public_key, derive_revocation_pubkey, signature_to_bitcoin_vec,
+};
 use crate::util::status::Status;
 use crate::util::INITIAL_COMMITMENT_NUMBER;
-use bitcoin::secp256k1::recovery::RecoverableSignature;
-use crate::IOError;
-use std::convert::TryInto;
+use crate::Arc;
+use lightning::ln::script::ShutdownScript;
+use lightning::ln::chan_utils::ClosingTransaction;
 
 /// Adapt MySigner to KeysInterface
 pub struct LoopbackSignerKeysInterface {
     pub node_id: PublicKey,
     pub signer: Arc<MultiSigner>,
-    pub backing: KeysManager,
 }
 
 impl LoopbackSignerKeysInterface {
     fn get_node(&self) -> Arc<Node> {
-        self.signer.get_node(&self.node_id)
+        self.signer
+            .get_node(&self.node_id)
             .expect("our node is missing")
     }
-}
 
+    pub fn spend_spendable_outputs(
+        &self,
+        descriptors: &[&SpendableOutputDescriptor],
+        outputs: Vec<TxOut>,
+        change_destination_script: Script,
+        feerate_sat_per_1000_weight: u32,
+        secp_ctx: &Secp256k1<All>,
+    ) -> Result<Transaction, ()> {
+        self.get_node().spend_spendable_outputs(
+            descriptors,
+            outputs,
+            change_destination_script,
+            feerate_sat_per_1000_weight,
+            secp_ctx,
+        )
+    }
+}
 
 #[derive(Clone)]
 pub struct LoopbackChannelSigner {
@@ -57,15 +80,14 @@ impl LoopbackChannelSigner {
         is_outbound: bool,
         channel_value_sat: u64,
     ) -> LoopbackChannelSigner {
-        log_info!(signer, "new channel {:?} {:?}", node_id, channel_id);
-        let pubkeys = signer.with_channel_base(&node_id, &channel_id, |base| {
-            Ok(base.get_channel_basepoints())
-        })
+        info!("new channel {:?} {:?}", node_id, channel_id);
+        let pubkeys = signer
+            .with_channel_base(&node_id, &channel_id, |base| {
+                Ok(base.get_channel_basepoints())
+            })
             .map_err(|s| {
-                // BEGIN NOT TESTED
-                log_error!(signer, "bad status {:?} on channel {}", s, channel_id);
+                error!("bad status {:?} on channel {}", s, channel_id);
                 ()
-                // END NOT TESTED
             })
             .expect("must be able to get basepoints");
         LoopbackChannelSigner {
@@ -100,22 +122,17 @@ impl LoopbackChannelSigner {
         Ok(keys)
     }
 
-    // BEGIN NOT TESTED
     fn bad_status(&self, s: Status) {
-        let signer = &self.signer;
-        log_error!(signer, "bad status {:?} on channel {}", s, self.channel_id);
+        error!("bad status {:?} on channel {}", s, self.channel_id);
     }
-    // END NOT TESTED
 
     fn sign_holder_commitment_and_htlcs(
         &self,
         hct: &HolderCommitmentTransaction,
     ) -> Result<(Signature, Vec<Signature>), ()> {
-        let signer = &self.signer;
         let commitment_tx = hct.trust();
 
-        log_debug!(
-            signer,
+        debug!(
             "loopback: sign local txid {}",
             commitment_tx.built_transaction().txid
         );
@@ -127,8 +144,9 @@ impl LoopbackChannelSigner {
         let (offered_htlcs, received_htlcs) =
             LoopbackChannelSigner::convert_to_htlc_info2(hct.htlcs());
 
-        let (sig_vec, htlc_sig_vecs) =
-            self.signer.with_ready_channel(&self.node_id, &self.channel_id, |chan| {
+        let (sig_vec, htlc_sig_vecs) = self
+            .signer
+            .with_ready_channel(&self.node_id, &self.channel_id, |chan| {
                 let result = chan.sign_holder_commitment_tx_phase2(
                     commitment_number,
                     feerate_per_kw,
@@ -138,7 +156,8 @@ impl LoopbackChannelSigner {
                     received_htlcs.clone(),
                 )?;
                 Ok(result)
-            }).map_err(|s| self.bad_status(s))?;
+            })
+            .map_err(|s| self.bad_status(s))?;
 
         let htlc_sigs = htlc_sig_vecs
             .iter()
@@ -167,50 +186,78 @@ impl LoopbackChannelSigner {
         }
         (offered_htlcs, received_htlcs)
     }
-
-    fn get_node(&self) -> Arc<Node> {
-        self.signer.get_node(&self.node_id)
-            .expect("our node is missing")
-    }
 }
 
-// BEGIN NOT TESTED
 impl Writeable for LoopbackChannelSigner {
     fn write<W: Writer>(&self, _writer: &mut W) -> Result<(), IOError> {
         unimplemented!()
     }
 }
-// END NOT TESTED
 
 fn bitcoin_sig_to_signature(mut res: Vec<u8>) -> Result<Signature, ()> {
     res.pop();
     let sig = Signature::from_der(res.as_slice())
-        .map_err(|_e| ()) // NOT TESTED
+        .map_err(|_e| ())
         .expect("failed to parse the signature we just created");
     Ok(sig)
 }
 
 impl BaseSign for LoopbackChannelSigner {
     fn get_per_commitment_point(&self, idx: u64, _secp_ctx: &Secp256k1<All>) -> PublicKey {
-        // signer layer expect forward counting commitment number, but we are passed a backwards counting one
-        self.signer.with_channel_base(&self.node_id, &self.channel_id, |base| {
-            Ok(base.get_per_commitment_point(INITIAL_COMMITMENT_NUMBER - idx))
-        })
+        // signer layer expect forward counting commitment number, but
+        // we are passed a backwards counting one
+        self.signer
+            .with_channel_base(&self.node_id, &self.channel_id, |base| {
+                Ok(base
+                    .get_per_commitment_point(INITIAL_COMMITMENT_NUMBER - idx)
+                    .unwrap())
+            })
             .map_err(|s| self.bad_status(s))
             .unwrap()
     }
 
     fn release_commitment_secret(&self, commitment_number: u64) -> [u8; 32] {
-        // signer layer expect forward counting commitment number, but we are passed a backwards counting one
-        let secret = self.signer.with_ready_channel(&self.node_id, &self.channel_id, |chan| {
-            let secret = chan.get_per_commitment_secret(INITIAL_COMMITMENT_NUMBER - commitment_number)[..]
-                .try_into()
-                .unwrap();
-            Ok(secret)
-        });
+        // signer layer expect forward counting commitment number, but
+        // we are passed a backwards counting one
+        let secret = self
+            .signer
+            .with_ready_channel(&self.node_id, &self.channel_id, |chan| {
+                let secret = chan
+                    .get_per_commitment_secret(INITIAL_COMMITMENT_NUMBER - commitment_number)
+                    .unwrap()[..]
+                    .try_into()
+                    .unwrap();
+                Ok(secret)
+            });
         secret.expect("missing channel")
     }
 
+    fn validate_holder_commitment(
+        &self,
+        holder_tx: &HolderCommitmentTransaction,
+    ) -> Result<(), ()> {
+        let commitment_number = INITIAL_COMMITMENT_NUMBER - holder_tx.commitment_number();
+        self.signer
+            .with_ready_channel(&self.node_id, &self.channel_id, |chan| {
+                let (offered_htlcs, received_htlcs) =
+                    LoopbackChannelSigner::convert_to_htlc_info2(holder_tx.htlcs());
+                chan.validate_holder_commitment_tx_phase2(
+                    commitment_number,
+                    holder_tx.feerate_per_kw(),
+                    holder_tx.to_broadcaster_value_sat(),
+                    holder_tx.to_countersignatory_value_sat(),
+                    offered_htlcs,
+                    received_htlcs,
+                    &holder_tx.counterparty_sig,
+                    &holder_tx.counterparty_htlc_sigs,
+                )?;
+                Ok(())
+            })
+            .map_err(|s| self.bad_status(s))?;
+
+        Ok(())
+    }
+
     fn pubkeys(&self) -> &ChannelPublicKeys {
         &self.pubkeys
     }
@@ -225,10 +272,8 @@ impl BaseSign for LoopbackChannelSigner {
         commitment_tx: &CommitmentTransaction,
         _secp_ctx: &Secp256k1<All>,
     ) -> Result<(Signature, Vec<Signature>), ()> {
-        let signer = &self.signer;
         let trusted_tx = commitment_tx.trust();
-        log_info!(
-            signer,
+        info!(
             "sign_counterparty_commitment {:?} {:?} txid {}",
             self.node_id,
             self.channel_id,
@@ -246,17 +291,20 @@ impl BaseSign for LoopbackChannelSigner {
         let to_counterparty_value_sat = commitment_tx.to_broadcaster_value_sat();
         let feerate_per_kw = commitment_tx.feerate_per_kw();
 
-        let (sig_vec, htlc_sigs_vecs) = self.signer.with_ready_channel(&self.node_id, &self.channel_id, |chan| {
-            chan.sign_counterparty_commitment_tx_phase2(
-                &per_commitment_point,
-                commitment_number,
-                feerate_per_kw,
-                to_holder_value_sat,
-                to_counterparty_value_sat,
-                offered_htlcs.clone(),
-                received_htlcs.clone(),
-            )
-        }).map_err(|s| self.bad_status(s))?;
+        let (sig_vec, htlc_sigs_vecs) = self
+            .signer
+            .with_ready_channel(&self.node_id, &self.channel_id, |chan| {
+                chan.sign_counterparty_commitment_tx_phase2(
+                    &per_commitment_point,
+                    commitment_number,
+                    feerate_per_kw,
+                    to_holder_value_sat,
+                    to_counterparty_value_sat,
+                    offered_htlcs.clone(),
+                    received_htlcs.clone(),
+                )
+            })
+            .map_err(|s| self.bad_status(s))?;
         let commitment_sig = bitcoin_sig_to_signature(sig_vec)?;
         let mut htlc_sigs = Vec::with_capacity(commitment_tx.htlcs().len());
         for htlc_sig_vec in htlc_sigs_vecs {
@@ -265,6 +313,17 @@ impl BaseSign for LoopbackChannelSigner {
         Ok((commitment_sig, htlc_sigs))
     }
 
+    fn validate_counterparty_revocation(&self, idx: u64, secret: &SecretKey) -> Result<(), ()> {
+        let forward_idx = INITIAL_COMMITMENT_NUMBER - idx;
+        self.signer
+            .with_ready_channel(&self.node_id, &self.channel_id, |chan| {
+                chan.validate_counterparty_revocation(forward_idx, secret)
+            })
+            .map_err(|s| self.bad_status(s))?;
+
+        Ok(())
+    }
+
     fn sign_holder_commitment_and_htlcs(
         &self,
         hct: &HolderCommitmentTransaction,
@@ -283,18 +342,17 @@ impl BaseSign for LoopbackChannelSigner {
             .with_ready_channel(&self.node_id, &self.channel_id, |chan| {
                 chan.keys
                     .unsafe_sign_holder_commitment_and_htlcs(hct, secp_ctx)
-                    .map_err(|_| Status::internal("could not unsafe-sign")) // NOT TESTED
+                    .map_err(|_| Status::internal("could not unsafe-sign"))
             })
-            .map_err(|_s| ()) // NOT TESTED
+            .map_err(|_s| ())
     }
 
-    fn sign_justice_transaction(
+    fn sign_justice_revoked_output(
         &self,
         justice_tx: &Transaction,
         input: usize,
         amount: u64,
         per_commitment_key: &SecretKey,
-        htlc: &Option<HTLCOutputInCommitment>,
         secp_ctx: &Secp256k1<All>,
     ) -> Result<Signature, ()> {
         let per_commitment_point = PublicKey::from_secret_key(secp_ctx, per_commitment_key);
@@ -306,29 +364,58 @@ impl BaseSign for LoopbackChannelSigner {
             counterparty_pubkeys,
             &self.pubkeys,
         )?;
-        let redeem_script = if let Some(ref htlc) = *htlc {
-            // BEGIN NOT TESTED
-            let tx_keys = self.make_counterparty_tx_keys(&per_commitment_point, secp_ctx)?;
-            chan_utils::get_htlc_redeemscript(&htlc, &tx_keys)
-        // END NOT TESTED
-        } else {
-            chan_utils::get_revokeable_redeemscript(
-                &revocation_key,
-                self.local_to_self_delay,
-                &delayed_payment_key,
-            )
-        };
+        let redeem_script = chan_utils::get_revokeable_redeemscript(
+            &revocation_key,
+            self.local_to_self_delay,
+            &delayed_payment_key,
+        );
 
         // TODO phase 2
-        let res = self.signer.with_ready_channel(&self.node_id, &self.channel_id, |chan| {
-            let sig = chan.sign_justice_sweep(
-                justice_tx,
-                input,
-                per_commitment_key,
-                &redeem_script,
-                amount)?;
-            Ok(signature_to_bitcoin_vec(sig))
-        }).map_err(|s| self.bad_status(s))?;
+        let res = self
+            .signer
+            .with_ready_channel(&self.node_id, &self.channel_id, |chan| {
+                let sig = chan.sign_justice_sweep(
+                    justice_tx,
+                    input,
+                    per_commitment_key,
+                    &redeem_script,
+                    amount,
+                )?;
+                Ok(signature_to_bitcoin_vec(sig))
+            })
+            .map_err(|s| self.bad_status(s))?;
+
+        bitcoin_sig_to_signature(res)
+    }
+
+    fn sign_justice_revoked_htlc(
+        &self,
+        justice_tx: &Transaction,
+        input: usize,
+        amount: u64,
+        per_commitment_key: &SecretKey,
+        htlc: &HTLCOutputInCommitment,
+        secp_ctx: &Secp256k1<All>,
+    ) -> Result<Signature, ()> {
+        let per_commitment_point = PublicKey::from_secret_key(secp_ctx, per_commitment_key);
+
+        let tx_keys = self.make_counterparty_tx_keys(&per_commitment_point, secp_ctx)?;
+        let redeem_script = chan_utils::get_htlc_redeemscript(&htlc, &tx_keys);
+
+        // TODO phase 2
+        let res = self
+            .signer
+            .with_ready_channel(&self.node_id, &self.channel_id, |chan| {
+                let sig = chan.sign_justice_sweep(
+                    justice_tx,
+                    input,
+                    per_commitment_key,
+                    &redeem_script,
+                    amount,
+                )?;
+                Ok(signature_to_bitcoin_vec(sig))
+            })
+            .map_err(|s| self.bad_status(s))?;
 
         bitcoin_sig_to_signature(res)
     }
@@ -346,16 +433,19 @@ impl BaseSign for LoopbackChannelSigner {
         let redeem_script = chan_utils::get_htlc_redeemscript(htlc, &chan_keys);
 
         // TODO phase 2
-        let res =
-            self.signer.with_ready_channel(&self.node_id, &self.channel_id, |chan| {
+        let res = self
+            .signer
+            .with_ready_channel(&self.node_id, &self.channel_id, |chan| {
                 let sig = chan.sign_counterparty_htlc_sweep(
                     htlc_tx,
                     input,
                     per_commitment_point,
                     &redeem_script,
-                    amount)?;
+                    amount,
+                )?;
                 Ok(signature_to_bitcoin_vec(sig))
-            }).map_err(|s| self.bad_status(s))?;
+            })
+            .map_err(|s| self.bad_status(s))?;
 
         bitcoin_sig_to_signature(res)
     }
@@ -363,51 +453,30 @@ impl BaseSign for LoopbackChannelSigner {
     // TODO - Couldn't this return a declared error signature?
     fn sign_closing_transaction(
         &self,
-        closing_tx: &Transaction,
+        closing_tx: &ClosingTransaction,
         _secp_ctx: &Secp256k1<All>,
     ) -> Result<Signature, ()> {
-        let signer = &self.signer;
-        log_info!(
-            signer,
+        info!(
             "sign_closing_transaction {:?} {:?}",
-            self.node_id,
-            self.channel_id
+            self.node_id, self.channel_id
         );
-        let mut to_holder_value = 0;
-        let mut to_counterparty_value = 0;
-        let local_script = payload_for_p2wpkh(
-            &self.get_node().get_shutdown_pubkey()
-        ).script_pubkey();
-        let mut to_counterparty_script = Script::default();
-        for out in &closing_tx.output {
-            if out.script_pubkey == local_script {
-                if to_holder_value > 0 {
-                    // BEGIN NOT TESTED
-                    log_error!(signer, "multiple to_holder outputs");
-                    return Err(());
-                    // END NOT TESTED
-                }
-                to_holder_value = out.value;
-            } else {
-                if to_counterparty_value > 0 {
-                    // BEGIN NOT TESTED
-                    log_error!(signer, "multiple to_counterparty outputs");
-                    return Err(());
-                    // END NOT TESTED
-                }
-                to_counterparty_value = out.value;
-                to_counterparty_script = out.script_pubkey.clone();
-            }
-        }
 
         // TODO error handling is awkward
-        self.signer.with_ready_channel(&self.node_id, &self.channel_id, |chan| {
-            chan.sign_mutual_close_tx_phase2(
-                to_holder_value,
-                to_counterparty_value,
-                Some(to_counterparty_script.clone())
-            )
-        }).map_err(|_| ())
+        self.signer
+            .with_ready_channel(&self.node_id, &self.channel_id, |chan| {
+
+                // FIXME - this needs to be supplied
+                let holder_wallet_path_hint = vec![];
+
+                chan.sign_mutual_close_tx_phase2(
+                    closing_tx.to_holder_value_sat(),
+                    closing_tx.to_counterparty_value_sat(),
+                    &Some(closing_tx.to_holder_script().clone()),
+                    &Some(closing_tx.to_counterparty_script().clone()),
+                    &holder_wallet_path_hint,
+                )
+            })
+            .map_err(|_| ())
     }
 
     fn sign_channel_announcement(
@@ -415,17 +484,17 @@ impl BaseSign for LoopbackChannelSigner {
         msg: &UnsignedChannelAnnouncement,
         _secp_ctx: &Secp256k1<All>,
     ) -> Result<Signature, ()> {
-        let signer = &self.signer;
-        log_info!(
-            signer,
+        info!(
             "sign_counterparty_commitment {:?} {:?}",
-            self.node_id,
-            self.channel_id
+            self.node_id, self.channel_id
         );
 
-        let (_nsig, bsig) = self.signer.with_ready_channel(&self.node_id, &self.channel_id, |chan| {
-            Ok(chan.sign_channel_announcement(&msg.encode()))
-        }).map_err(|s| self.bad_status(s))?;
+        let (_nsig, bsig) = self
+            .signer
+            .with_ready_channel(&self.node_id, &self.channel_id, |chan| {
+                Ok(chan.sign_channel_announcement(&msg.encode()))
+            })
+            .map_err(|s| self.bad_status(s))?;
 
         let res = bsig.serialize_der().to_vec();
 
@@ -435,12 +504,9 @@ impl BaseSign for LoopbackChannelSigner {
     }
 
     fn ready_channel(&mut self, parameters: &ChannelTransactionParameters) {
-        let signer = &self.signer;
-        log_info!(
-            signer,
+        info!(
             "set_remote_channel_pubkeys {:?} {:?}",
-            self.node_id,
-            self.channel_id
+            self.node_id, self.channel_id
         );
 
         // TODO cover local vs remote to_self_delay with a test
@@ -449,19 +515,19 @@ impl BaseSign for LoopbackChannelSigner {
         let setup = ChannelSetup {
             is_outbound: self.is_outbound,
             channel_value_sat: self.channel_value_sat,
-            push_value_msat: 0, // TODO
+            push_value_msat: 1_000_000, // TODO
             funding_outpoint,
-            holder_to_self_delay: parameters.holder_selected_contest_delay,
+            holder_selected_contest_delay: parameters.holder_selected_contest_delay,
             holder_shutdown_script: None, // use the signer's shutdown script
             counterparty_points: counterparty_parameters.pubkeys.clone(),
-            counterparty_to_self_delay: counterparty_parameters.selected_contest_delay,
-            counterparty_shutdown_script: Default::default(), // TODO
+            counterparty_selected_contest_delay: counterparty_parameters.selected_contest_delay,
+            counterparty_shutdown_script: None, // TODO
             commitment_type: CommitmentType::StaticRemoteKey, // TODO
         };
-        let node = self.signer.get_node(&self.node_id)
-            .expect("no such node");
+        let node = self.signer.get_node(&self.node_id).expect("no such node");
 
-        node.ready_channel(self.channel_id, None, setup)
+        let holder_shutdown_key_path = vec![];
+        node.ready_channel(self.channel_id, None, setup, &holder_shutdown_key_path)
             .expect("channel already ready or does not exist");
         // Copy some parameters that we need here
         self.counterparty_pubkeys = Some(counterparty_parameters.pubkeys.clone());
@@ -484,14 +550,14 @@ impl KeysInterface for LoopbackSignerKeysInterface {
         self.get_node().get_destination_script()
     }
 
-    fn get_shutdown_pubkey(&self) -> PublicKey {
-        self.get_node().get_shutdown_pubkey()
+    fn get_shutdown_scriptpubkey(&self) -> ShutdownScript {
+        // FIXME - this method is deprecated
+        self.get_node().get_ldk_shutdown_scriptpubkey()
     }
 
     fn get_channel_signer(&self, is_inbound: bool, channel_value_sat: u64) -> Self::Signer {
         let node = self.signer.get_node(&self.node_id).unwrap();
-        let (channel_id, _) =
-            node.new_channel(None, None, &node).unwrap();
+        let (channel_id, _) = node.new_channel(None, None, &node).unwrap();
         LoopbackChannelSigner::new(
             &self.node_id,
             &channel_id,
@@ -502,10 +568,9 @@ impl KeysInterface for LoopbackSignerKeysInterface {
     }
 
     fn get_secure_random_bytes(&self) -> [u8; 32] {
-        self.backing.get_secure_random_bytes()
+        self.get_node().get_secure_random_bytes()
     }
 
-    // BEGIN NOT TESTED
     fn read_chan_signer(&self, _reader: &[u8]) -> Result<Self::Signer, DecodeError> {
         unimplemented!()
     }
@@ -513,7 +578,6 @@ impl KeysInterface for LoopbackSignerKeysInterface {
     fn sign_invoice(&self, invoice_preimage: Vec<u8>) -> Result<RecoverableSignature, ()> {
         Ok(self.get_node().sign_invoice(&invoice_preimage))
     }
-    // END NOT TESTED
 }
 
 fn get_delayed_payment_keys(
